@@ -9,16 +9,16 @@ use Carp;
 use File::Spec;
 use IO::Dir;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 our @DIRS;
-our %TYPES;
+our %ALREADY;
 
 require Exporter;
 
 our @ISA = qw( Exporter );
-our @EXPORT = qw( install_share );
-our @EXPORT_OK = qw( postamble install_share );
+our @EXPORT = qw( install_share delete_share );
+our @EXPORT_OK = qw( postamble install_share delete_share );
 our $INCLUDE_DOTFILES = 0;
 our $INCLUDE_DOTDIRS = 0;
 
@@ -27,40 +27,102 @@ sub install_share
 {
     my $dir  = @_ ? pop : 'share';
     my $type = @_ ? shift : 'dist';
-    unless ( defined $type and $type eq 'module' or $type eq 'dist' ) {
+    unless ( defined $type and 
+            ( $type =~ /^(module|dist)$/ ) ) {
         confess "Illegal or invalid share dir type '$type'";
     }
-    unless ( defined $dir and -d $dir ) {
-        confess "Illegal or missing directory '$dir'";
-    }
 
     if( $type eq 'dist' and @_ ) {
-        confess "Too many parameters to share_dir";
+        confess "Too many parameters to install_share";
     }
 
-    push @DIRS, $dir;
-    $TYPES{$dir} = { type=>$type, 
-                     dotfiles => $INCLUDE_DOTFILES, 
-                     dotdirs => $INCLUDE_DOTDIRS
-                    };
-    if( $type eq 'module' ) {
-        my $module = _CLASS( $_[0] );
+    my $def = _mk_def( $type );
+    _add_module( $def, $_[0] );
+
+    _add_dir( $def, $dir );
+}
+
+
+#####################################################################
+sub delete_share
+{
+    my $dir  = @_ ? pop : '';
+    my $type = @_ ? shift : 'dist';
+    unless ( defined $type and 
+            ( $type =~ /^(module|dist)$/ ) ) {
+        confess "Illegal or invalid share dir type '$type'";
+    }
+
+    if( $type eq 'dist' and @_ ) {
+        confess "Too many parameters to delete_share";
+    }
+
+    my $def = _mk_def( "delete-$type" );
+    _add_module( $def, $_[0] );
+    _add_dir( $def, $dir );
+}   
+
+
+
+#
+# Build a task definition
+sub _mk_def
+{
+    my( $type ) = @_;
+    return { type=>$type, 
+             dotfiles => $INCLUDE_DOTFILES, 
+             dotdirs => $INCLUDE_DOTDIRS
+           };
+}
+
+#
+# Add the module to a task definition
+sub _add_module
+{
+    my( $def, $class ) =  @_;
+    if( $def->{type} =~ /module$/ ) {
+        my $module = _CLASS( $class );
         unless ( defined $module ) {
             confess "Missing or invalid module name '$_[0]'";
         }
-        $TYPES{$dir}{module} = $module;
+        $def->{module} = $module;
     }
-
 }
 
+#
+# Add directories to a task definition
+# Save the definition
+sub _add_dir
+{
+    my( $def, $dir ) = @_;
+
+    $dir = [ $dir ] unless ref $dir;
+
+    my $del = 0;
+    $del = 1 if $def->{type} =~ /^delete-/;
+
+    foreach my $d ( @$dir ) {
+        unless ( $del or (defined $d and -d $d) ) {
+            confess "Illegal or missing directory '$d'";
+        }
+        if( not $del and $ALREADY{ $d }++ ) {
+            confess "Directory '$d' is already being installed";
+        }
+        push @DIRS, { %$def };
+        $DIRS[-1]{dir} = $d;
+    }
+}   
+
+
 #####################################################################
+# Build the postamble section
 sub postamble 
 {
     my $self = shift;
 
     my @ret; # = $self->SUPER::postamble( @_ );
-    foreach my $dir ( @DIRS ) {
-        push @ret, __postamble_share_dir( $self, $dir, $TYPES{ $dir } );
+    foreach my $def ( @DIRS ) {
+        push @ret, __postamble_share_dir( $self, $def );
     }
     return join "\n", @ret;
 }
@@ -68,33 +130,40 @@ sub postamble
 #####################################################################
 sub __postamble_share_dir
 {
-    my( $self, $dir, $def ) = @_;
+    my( $self, $def ) = @_;
 
+    my $dir = $def->{dir};
+
+    $DB::single = 1;
     my( $idir );
-    if ( $def->{type} eq 'dist' ) {
-        $idir = File::Spec->catdir( '$(INST_LIB)', 
-                                    qw( auto share dist ), 
-                                    '$(DISTNAME)'
-                                  );
+
+    if( $def->{type} eq 'delete-dist' ) {
+        $idir = File::Spec->catdir( _dist_dir(), $dir );
     } 
-    else {
-        my $module = $def->{module};
-        $module =~ s/::/-/g;
-        $idir = File::Spec->catdir( '$(INST_LIB)', 
-                                    qw( auto share module ), 
-                                    $module
-                                  );
+    elsif( $def->{type} eq 'delete-module' ) {
+        $idir = File::Spec->catdir( _module_dir( $def ), $dir );
+    }
+    elsif ( $def->{type} eq 'dist' ) {    
+        $idir = _dist_dir();
+    } 
+    else {                                  # delete-share and share
+        $idir = _module_dir( $def );
     }
 
-    my $files = {};
-    _scan_share_dir( $files, $idir, $dir, $def );
-
-    my $autodir = '$(INST_LIB)';
-    my $pm_to_blib = $self->oneliner(<<CODE, ['-MExtUtils::Install']);
+    my @cmds;
+    if( $def->{type} =~ /^delete-/ ) {
+        @cmds = "\$(RM_RF) $idir";
+    }
+    else {
+        my $autodir = '$(INST_LIB)';
+        my $pm_to_blib = $self->oneliner(<<CODE, ['-MExtUtils::Install']);
 pm_to_blib({\@ARGV}, '$autodir')
 CODE
 
-    my @cmds = $self->split_command( $pm_to_blib, %$files );
+        my $files = {};
+        _scan_share_dir( $files, $idir, $dir, $def );
+        @cmds = $self->split_command( $pm_to_blib, %$files );
+    }
 
     my $r = join '', map { "\t\$(NOECHO) $_\n" } @cmds;
 
@@ -104,6 +173,28 @@ CODE
     return "config::\n$r";
 }
 
+# Get the per-dist install directory.
+# We depend on the Makefile for most of the info
+sub _dist_dir
+{
+    return File::Spec->catdir( '$(INST_LIB)', 
+                                    qw( auto share dist ), 
+                                    '$(DISTNAME)'
+                                  );
+}
+
+# Get the per-module install directory
+# We depend on the Makefile for most of the info
+sub _module_dir
+{
+    my( $def ) = @_;
+    my $module = $def->{module};
+    $module =~ s/::/-/g;
+    return  File::Spec->catdir( '$(INST_LIB)', 
+                                    qw( auto share module ), 
+                                    $module
+                                  );
+}
 
 sub _scan_share_dir
 {
@@ -189,14 +280,18 @@ Causes all the files in C<$dir> and its sub-directories to be installed
 into a per-dist or per-module share directory.  Must be called before
 L<WriteMakefile>.
 
-The first 2 forms are equivalent.
+The first 2 forms are equivalent; the files are installed in a per-distribution
+directory.  For example C</usr/lib/perl5/site_perl/auto/share/dist/My-Dist>.  The
+name of that directory can be recovered with L<File::ShareDir/dist_dir>.
 
-The files will be installed when you run C<make install>.
+The last form installs files in a per-module directory.  For example 
+C</usr/lib/perl5/site_perl/auto/share/module/My-Dist-Package>.  The name of that
+directory can be recovered with L<File::ShareDir/module_dir>.
 
-To locate the files after installation so they can be used inside your
-module, see  L<File::ShareDir>.
+The parameter C<$dir> may be an array of directories.
 
-    my $dir = File::ShareDir::module_dir( $module );
+The files will be installed when you run C<make install>.  However, the list
+of files to install is generated when Makefile.PL is run.
 
 Note that if you make multiple calls to C<install_share> on different
 directories that contain the same filenames, the last of these calls takes
@@ -205,13 +300,45 @@ precedence.  In other words, if you do:
     install_share 'share1';
     install_share 'share2';
 
-And both C<share1> and C<share2> contain a fill called C<info>, the file
-C<share2/info> will be installed into your C<dist_dir()>.
+And both C<share1> and C<share2> contain a file called C<info.txt>, the file
+C<share2/info.txt> will be installed into your C<dist_dir()>.
+
+=head2 delete_share
+
+    delete_share $list;
+    delete_share dist => $list;
+    delete_share module => $module, $list;
+    
+Remove previously installed files or directories.  
+
+Unlike L</install_share>, the last parameter is a list of files or
+directories that were previously installed.  These files and directories will
+be deleted when you run C<make install>.
+
+The parameter C<$list> may be an array of files or directories.
+
+Deletion happens in-order along with installation.  This means that you may
+delete all previously installed files by putting the following at the top of
+your Makefile.PL.
+
+    delete_share '.';
+
+You can also selectively remove some files from installation.
+
+    install_share 'some-dir';
+    if( ... ) {
+        delete_share 'not-this-file.rc';
+    }
 
 =head2 postamble
 
-Exported into the MY package.  Only documented here if you need to write your
-own postamble.
+This function must be exported into the MY package.  You will normaly do this
+with the following.
+
+    package MY;
+    use File::ShareDir::Install qw( postamble );
+
+If you need to overload postamble, use the following.
 
     package MY;
     use File::ShareDir::Install;
@@ -228,8 +355,9 @@ own postamble.
 2 variables control the handling of dot-files and dot-directories.
 
 A dot-file has a filename that starts with a period (.).  For example
-C<.htaccess> A dot-directory (or dot-dir) is a directory that starts with a
-period (.).  For example C<.config/>.  Not all OSes support the use of dot-files.
+C<.htaccess>. A dot-directory (or dot-dir) is a directory that starts with a
+period (.).  For example C<.config/>.  Not all filesystems support the use
+of dot-files.
 
 =head2 $INCLUDE_DOTFILES
 
@@ -242,7 +370,7 @@ Known version control directories are still ignored.  Default is false.
 
 =head2 Note
 
-These variables only influences subsequent calls to C<install_share()>.  This allows
+These variables only influence subsequent calls to C<install_share()>.  This allows
 you to control the behaviour for each directory.  
 
 For example:
@@ -270,7 +398,7 @@ Philip Gwyn, E<lt>gwyn-AT-cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009-2011 by Philip Gwyn
+Copyright (C) 2009-2014 by Philip Gwyn
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
